#!/usr/bin/env python
#-*- coding: iso8859-15 -*-
##########################################################
#                  FILE_PESYR v1.0                       #
##########################################################
# Autor: Juan Miguel Taboada Godoy                       #
# Fecha: Szczecin, 28 de Julio de 2006                   #
# Descripcin: Cargador de datos de fichero de PESYR     #
# Versin: 2006071900                                    #
#                                                        #
# Codigo fuente bajo licencia GNU/GPL                    #
# Centrologic (Computational Logistic Center)            #
# http://www.centrologic.com - info@centrologic.com      #
##########################################################

# Libreras necesarias {{{1
import gzip
import fcntl
from FILTER import coma2point
from DATE import timenormal
from HERENCIA import BASE
from INTERFACES import READABLE,LockError
from MATH_BIN import hextobin
# }}}1
# Funciones generales {{{1
# Funcion para obtener el listado de palabras de una cadena {{{2
def palabras (cadena):
	cad=''
	trozos=[]
	for caracter in cadena:
		if caracter=='!':
			trozos.append(cad)
			cad=''
		elif (caracter=='\n') or (caracter=='\r'):
			cad=cad
		else:
			cad=cad+caracter
	trozos.append(cad)
	cad=''
	return trozos
#}}}2
# }}}1

# ### FILE_PESYR ### ##################################
## \internal
## FILE_PESYR: Lector de ficheros con formato PESYR
## \version 19/07/2006 1600 Malaga
class FILE_PESYR(BASE,READABLE):
	# Constructor {{{1
	def __init__(self):
		# Cargo las acciones del padre
		BASE.__init__(self,None,"FILE_PESYR")
		# Inicializo el contador de elementos
		self.__elemento_actual=0
		# Inicializo la frecuencia interna
		self.__frecuencia=None
		# Inicializo la compresion
		self.__compress=False
		# Inicializa a None el mantener bloqueo
		self.__keeplock=None
		# Creo la lista vaca
		self.__lista=[]
	# }}}1
	# Destructor {{{1
	## Destructor
	## \param - self
	def __del__(self):
		# Close the file if is locked with keeplock
		self.close()
	# }}}1

	# Devuelve si el fichero esta comprimido o no {{{1
	## Devuelve si el fichero esta comprimido o no
	## \param self -
	def compress(self):
		return self.__compress
	# }}}1
	# Frecuencia de compresion interna {{{1
	## Frecuencia de compresion interna
	## \param self -
	## \param value Valor de la frecuencia de compresion interna (opcional)
	def frecuencia(self,value=None):
		if (value==None):
			return self.__frecuencia
		else:
			self.__frecuencia=value
	# }}}1
	# Aade un elemento a la lista {{{1
	## \internal
	## Inserta un elemento en la lista de datos
	## \param self -
	## \param id ID de la senal
	## \param date Fecha y hora
	## \param value Valor contenido
	## \return True si lo ha insertado, False si no lo ha insertado
	def __add(self,id,date,value):
		# Comprueba la condiccin de carga
		if (value!=""):
			# Aade el elemento a la lista
			self.__lista.append((id,date,value))
			# Devuelve que si lo aadi
			return True
		else:
			# Devuelve que no lo aadi
			return False
	# }}}1
	# Carga el fichero de datos ordenadamente {{{1
	## \internal
	## Carga el fichero de datos ordenadamente en la memoria
	## \param self -
	## \param fichero Fichero a leer
	## \param flock nolock:No bloquear, wait:esperar a que se libere el fichero, lock:bloquear o fallar (Por defecto: nolock)
	## \param keeplock Indica si mantener el bloqueo despues del load (Por defecto: False)
	def load(self,fichero,flock="nolock",keeplock=False):
		self.debug("Cargando el fichero %s" % (fichero))
		# Revisa si la extension del fichero es gz
		if (fichero.split(".")[-1]=="gz"):
			# En tal caso abro el fichero con el modulo gzip
			file=gzip.open(fichero,"r")
			# Bloqueo el acceso al fichero si necesario
			if (flock=="wait"):
				fcntl.flock(file.fileno(), fcntl.LOCK_EX)
			elif (flock=="lock"):
				try:
					fcntl.flock(file.fileno(), fcntl.LOCK_EX|fcntl.LOCK_NB)
				except IOError,error:
					raise LockError,error
			# Recuerdo que est comprimido
			self.__compress=True
		else:
			# Si es un fichero normal lo abro normalmente
			file=open(fichero,"r")
			# Bloqueo el acceso al fichero si necesario
			if (flock=="wait"):
				fcntl.flock(file.fileno(), fcntl.LOCK_EX)
			elif (flock=="lock"):
				try:
					fcntl.flock(file.fileno(), fcntl.LOCK_EX|fcntl.LOCK_NB)
				except IOError,error:
					raise LockError,error
			# Recuerdo que no est comprimido
			self.__compress=False
		# Etiqueta 1
		self.debug("Leyendo etiqueta 1")
		file.readline()
		# Cabeceras
		self.debug("Cabeceras")
		# medio=
		file.readline()
		# telefono=
		file.readline()
		# producto=
		producto=file.readline().split("=")[1][0:2]
		# montaje=
		file.readline()
		# serie=
		seriebig=file.readline()[0:-1].split("=")[1]
		serie=seriebig[0]
		if (producto=="IA" and serie=="4"):
			modelotelemando="l3"
		elif (producto=="IB" and serie=="5"):
			modelotelemando="l6"
		else:
			raise IOError,"Fichero corrupto, el producto (%s) y el indice (%s) del numero de serie (%s) no coinciden." % (producto,serie,seriebig)
		# version=
		file.readline()
		# servicio=
		file.readline()
		# Etiqueta 2
		self.debug("Leyendo etiqueta 2")
		file.readline()
		# Proceso el fichero {{{2
		self.debug("Procesando el fichero")
		contenido=file.readlines()
		for linea in contenido:
			# Inicializo los valores
			ea_med=[]
			ea_min=[]
			ea_max=[]
			ec=[]
			# Separo cada lnea
			s=palabras(linea)
			# Compruebo que tiene el nmero de palabras esperadas (Aadir tamaos aqu implica reconocerlos en la recogida de registros mas abajo)
			if ((len(s)==40) or (len(s)==41) or (len(s)==5) or (len(s)==73) or (len(s)==74)):
				# Debug
				# Recojo los datos genricos {{{3
				#tipo=s[0]					# Codigo de estado
				fechaSalvaje=s[2]			# Fecha salvaje
				# Adapto la fecha al formato requerido por la Base de Datos {{{4
				# Traduzco la Feche Salvaje a TIMENORMAL
				fecha=timenormal(fechaSalvaje)
				# Recorto la fecha para separar fecha y hora
				# longitud=len(fecha)
				lista=[]
				cad=''
				for caracter in fecha:
					if (caracter==' '):
						lista.append(cad)
						cad=''
					elif (caracter=='\n') or (caracter=='\r'):
						cad=cad
					else:
						cad=cad+caracter
				lista.append(cad)
				cad=''
				# Separo las variables
				fecha=lista[0]
				hora=lista[1]
				# }}}4
				# }}}3
				# Recojo el resto de registros {{{3
				if ((len(s)==40) or (len(s)==41) or (len(s)==73) or (len(s)==74)):
					# Recojo los datos de los registros tipo largo {{{4
					# Estado de las entradas digitales
					if (s[3]!=""):
						binario_ed=hextobin(s[3])
						if  (modelotelemando=="l6"):
							indexini=8
						else:
							indexini=0
						for i in range(indexini,16):
							self.__add("ed:%s" % (i),(fecha,hora),binario_ed[-i-1])
					# Estado de las entradas analgicas
					if (s[4]!=""):
						binario_ea=hextobin(s[4])
						for i in range(0,8):
							self.__add("ea:%s" % (i),(fecha,hora),binario_ea[-i-1])
					# Estado de las salidas de los rels
					if (s[5]!=""):
						binario_sd=hextobin(s[5])
						for i in range(0,8):
							self.__add("sd:%s" % (i),(fecha,hora),binario_sd[-i-1])
					# Estado de bits especiales
					if (s[6]!=""):
						binario_be=hextobin(s[6])
						for i in range(0,8):
							self.__add("be:%s" % (i),(fecha,hora),binario_be[-i-1])
					ea_med.append(s[7])		# Entrada analgica 0 [Media  del periodo]
					ea_min.append(s[8])		# Entrada analgica 0 [Mnimo del periodo]
					ea_max.append(s[9])		# Entrada analgica 0 [Mximo del periodo]
					ea_med.append(s[10])	# Entrada analgica 1 [Media  del periodo]
					ea_min.append(s[11])	# Entrada analgica 1 [Mnimo del periodo]
					ea_max.append(s[12])	# Entrada analgica 1 [Mximo del periodo]
					ea_med.append(s[13])	# Entrada analgica 2 [Media  del periodo]
					ea_min.append(s[14])	# Entrada analgica 2 [Mnimo del periodo]
					ea_max.append(s[15])	# Entrada analgica 2 [Mximo del periodo]
					ea_med.append(s[16])	# Entrada analgica 3 [Media  del periodo]
					ea_min.append(s[17])	# Entrada analgica 3 [Mnimo del periodo]
					ea_max.append(s[18])	# Entrada analgica 3 [Mximo del periodo]
					ea_med.append(s[19])	# Entrada analgica 4 [Media  del periodo]
					ea_min.append(s[20])	# Entrada analgica 4 [Mnimo del periodo]
					ea_max.append(s[21])	# Entrada analgica 4 [Mximo del periodo]
					ea_med.append(s[22])	# Entrada analgica 5 [Media  del periodo]
					ea_min.append(s[23])	# Entrada analgica 5 [Mnimo del periodo]
					ea_max.append(s[24])	# Entrada analgica 5 [Mximo del periodo]
					ea_med.append(s[25])	# Entrada analgica 6 [Media  del periodo]
					ea_min.append(s[26])	# Entrada analgica 6 [Mnimo del periodo]
					ea_max.append(s[27])	# Entrada analgica 6 [Mximo del periodo]
					ea_med.append(s[28])	# Entrada analgica 7 [Media  del periodo]
					ea_min.append(s[29])	# Entrada analgica 7 [Mnimo del periodo]
					ea_max.append(s[30])	# Entrada analgica 7 [Mximo del periodo]
					ec.append(s[31])		# Valor de conteo 0
					ec.append(s[32])		# Valor de conteo 1
					ec.append(s[33])		# Valor de conteo 2
					ec.append(s[34])		# Valor de conteo 3
					ec.append(s[35])		# Valor de conteo 4
					ec.append(s[36])		# Valor de conteo 5
					ec.append(s[37])		# Valor de conteo 6
					ec.append(s[38])		# Valor de conteo 7
					self.__add("cobertura",(fecha,hora),s[39])	# Cobertura GSM
					if (len(s)==41):
						self.__add("extra",(fecha,hora),s[40])	# Bits EXTRAs
					# }}}4
					# Reajusto los valores {{{4
					# MEDIA {{{5
					for i in range(len(ea_med)):
						if (ea_med[i]==""): ea_med[i]="NULL"		# MEDIA
						else: ea_med[i]=coma2point(ea_med[i])	# Intercambia , por . en Media
					# }}}5
					# MAXIMO {{{5
					for i in range(len(ea_max)):
						if (ea_max[i]==""): ea_max[i]="NULL"		# MAXIMO
						else: ea_max[i]=coma2point(ea_max[i])	# Intercambia , por . en Maximo
					# }}}5
					# MINIMO {{{5
					for i in range(len(ea_min)):
						if (ea_min[i]==""): ea_min[i]="NULL"		# MINIMO
						else: ea_min[i]=coma2point(ea_min[i])	# Intercambia , por . en Mnimo
					# }}}5
					# CONTEO {{{5
					for i in range(len(ec)):
						if ec[i]=="": ec[i]="NULL"					# CONTEO
					# }}}5
					# }}}4
					# Aado los registros {{{4
					for i in range(len(ea_med)): self.__add("eamed:%d" % (i),(fecha,hora),ea_med[i])
					for i in range(len(ea_min)): self.__add("eamin:%d" % (i),(fecha,hora),ea_min[i])
					for i in range(len(ea_max)): self.__add("eamax:%d" % (i),(fecha,hora),ea_max[i])
					for i in range(len(ec)): self.__add("ec:%d" % (i),(fecha,hora),ec[i])
					# }}}4
				elif (len(s)==5):
					# Recojo los registros cortos (eventos) {{{4
					self.__add("cobertura",(fecha,hora),s[4])	# Cobertura GSM
					# }}}4
				# }}}3
			else:
				# El nmero de palabras es incorrecto, la lnea no se usara
				self.debug("Cadena incorrecta en el fichero '%s', se rechaza por tener longitud no reconocida (%s elementos): %s" % (fichero,len(s),linea))
			
		# }}}2
		# Ordeno los resultados por fecha (La clase principal lo hace por si sola)
		#self.__lista.sort(lambda x,y:x[1]<y[1])
		# Cierro el fichero
		if (keeplock):
			self.__keeplock=file
		else:
			self.debug("Cerrando el fichero")
			file.close()
	# }}}1
	# Cierra el fichero cargado durante el load {{{1
	## Cierra el fichero cargado durante el load
	def close(self):
		if (self.__keeplock!=None):
			self.debug("Cerrando el fichero")
			self.__keeplock.close()
			self.__keeplock=None
	# }}}1
	# Devuelve el siguiente elemento de la lista {{{1
	## \internal
	## Devuelve el siguiente elemento almacenado en memoria
	## \param self -
	## \return Devuelve el elemento siguiente en la memoria o None si no hay mas elementos
	def next(self):
		if (self.__elemento_actual<len(self.__lista)):
			# Obtengo el elemento de la lista
			elemento=self.__lista[self.__elemento_actual]
			# Incremento en 1 el puntero de elemento_actual
			self.__elemento_actual=self.__elemento_actual+1
		else:
			# No hay ms elementos
			elemento=None
		# No hay ms elementos que leer
		return elemento
	# }}}1
	# Indica la longitud de la lista de elementos a procesar {{{1
	## \internal
	## Indica la longitud de la lista de elementos que hay en memoria
	## \param self -
	## \return Devuelve el numero de elementos que hay en la memoria
	def lenght(self):
		return len(self.__lista)
	# }}}1

