#!/usr/bin/env python
#-*- coding: iso8859-15 -*-
##########################################################
#               LIBRERIA DE GRAFICOS v1.0                #
##########################################################
# Autor: Juan Miguel Taboada Godoy                       #
# Fecha: Malaga, 16 de noviembre de 2006                 #
# Descripcion: Contiene todos las funciones matematicas  #
# Version: 2007060300                                    #
#                                                        #
# Codigo fuente bajo licencia GNU/GPL                    #
# Centrologic (Computational Logistic Center)            #
# http://www.centrologic.com - info@centrologic.com      #
##########################################################

# Librerias {{{1
from TAD import Stack
from MATH_GRA import calculadora
from MATH_EV import sqrtEV, escalarX, particionar
from FILTER import separar_operadores
# }}}1
# Funciones generales {{{1
# Encuentra la asignacion de color a un valor segun la matriz de pares rango-color {{{2
def find_color(rancol,valor):
	result=None
	for par in rancol:
		(ini,fin,color)=par
		if ((ini<=valor) and (valor<=fin)):
			result=color
			break
	return result
# }}}2
# }}}1

# Clase graficas para ser extendida
## Define las funciones basicas de un graficador
class GRAFICADOR_CONFIG:
	# Config Graficas {{{1
	# Conector con el lector de datos {{{2
	## Conector con el lector de datos
	## \param self -
	## \param arg Objeto de la clase src.lib.INTERFACES.READABLE que entregara los datos necesarios para las graficas
	def lector(self,arg=None):
		if (arg!=None):
			self._lector=arg
		else:
			return self._lector
	# }}}2
	# Test {{{2
	## Informacion sobre que datos realizar el test de informacion disponible en la grafica
	## \param self -
	## \param nombre Nombre de la senal sobre la que hacer el test (si es False el test siempre da positivo como resultado)
	## \param minX Desde que posicion en X realizar el test
	## \param maxX Hasta que posicion en X realizar el test
	## \param minY Desde que posicion en Y realizar el test
	## \param maxY Hata que posicion en Y realizar el test
	## \param linea Indica si el nombre dado en el primer parametro es una linea (Por defecto es False)
	## \attention El test debe encontrar almenos un dato en la caja definida por las cuatro esquinas: minX, maxX, minY y maxY
	def test(self,nombre=None,minX=None,maxX=None,minY=None,maxY=None,linea=False):
		if (nombre!=None):
			self._test=(nombre,minX,maxX,minY,maxY,linea)
		else:
			return self._test
	# }}}2
	# Memoria {{{2
	## Fichero que contiene la memoria
	## \param self -
	## \param arg Nombre del fichero
	def memoria(self,arg=None):
		if (arg!=None):
			self._memoria=arg
		return self._memoria
	# }}}2
	# Directorio de logs {{{2
	## Directorio donde guardar los logs
	## \param self -
	## \param arg Directorio
	def dirlog(self,arg=None):
		if (arg!=None):
			self._dirlog=arg
		return self._dirlog
	# }}}2
	# Cabecera del fichero de logs {{{2
	## Cabecera del fichero de logs
	## \param self -
	## \param arg Texto de cabecera del fichero
	def cabecera_log(self,arg=None):
		if (arg!=None):
			self._cabecera_log=arg
		return self._cabecera_log
	# }}}2
	# }}}1
	# Checker del Graficador {{{1
	## Comprueba que el conjunto minimo de datos requeridos por la clase han sido declarados por el usuario: \n
	## - lector()
	## - test()
	## - memoria()
	## \param self -
	def check_graficador(self):
		self._check("lector")
		self._check("test")
		self._check("memoria")
	# }}}1

# Clase graficas para ser extendida
## Define las funciones basicas de un graficador
class GRAFICADOR:
	# Constructor {{{1
	## Constructor
	## \param self -
	## \param nombre Nombre de la grafica
	## \param senales Objeto senales que contiene el listado completo de senales conocidas
	def __init__(self,nombre,senales):
		# Tabla de senales
		self.__tablasenales=senales
		# Tabla de lineas
		self.__tablalineas=[]
		# Tabla de ejes
		self.__axis=[]
		# Inicializo el nombre de la gráfica
		self.__nombre=nombre
		# Para viajes temporales de la configuracion
		self._viaje_temporal=None
	# }}}1

	# Permite consultar el nombre interno de la grafica {{{1
	## Permite consultar el nombre interno de la grafica
	## \param self -
	## \return Nombre interno de la grafica
	def nombre(self):
		return self.__nombre
	# }}}1
	# Permite consultar la tabla de senales {{{1
	## \internal
	## Permite consultar la tabla de senales
	## \param self -
	def tablasenales(self):
		return self.__tablasenales
	# }}}1
	# Permite consultar la tabla de lineas {{{1
	## \internal
	## Permite consultar la tabla de lineas
	## \param self -
	def tablalineas(self):
		return self.__tablalineas
	# }}}1
	# Permite consultar el valor de axis {{{1
	## Permite consultar el valor del eje
	## \param self -
	def axis(self):
		return self.__axis
	# }}}1
	# Permite anotar cual es el viaje temporal que va a sufrir la configuracion {{{1
	## Permite anotar cual es el viaje temporal que va a sufrir la configuracion
	## \param self -
	## \param arg Destino del viaje temporal (es una fecha)
	def viaje_temporal(self,arg=None):
		if (arg!=None):
			self._viaje_temporal=arg
		else:
			return self._viaje_temporal
	# }}}1
	# Permite anotar y recuperar el valor de NA {{{1
	## Funcion para controlar el valor de NA (lectura y escritura)
	## \param self -
	## \param arg Valor de NA
	def NA(self,arg=None):
		if (arg!=None):
			self._NA=arg
		else:
			# Si no esta declarada, la declaro como None
			if (not ("_NA" in vars(self))):
				self._NA=None
		return self._NA
	# }}}2
	# }}}1

	# Test {{{1
	## Informacion sobre que datos realizar el test de informacion disponible en la grafica
	## \param self -
	## \param nombre Nombre de la senal sobre la que hacer el test (si es False el test siempre da positivo como resultado)
	## \param minX Desde que posicion en X realizar el test
	## \param maxX Hasta que posicion en X realizar el test
	## \param minY Desde que posicion en Y realizar el test
	## \param maxY Hata que posicion en Y realizar el test
	## \param linea Indica si el nombre dado en el primer parametro es una linea (Por defecto es False)
	## \attention El test debe encontrar almenos un dato en la caja definida por las cuatro esquinas: minX, maxX, minY y maxY
	def test(self,nombre=None,minX=None,maxX=None,minY=None,maxY=None,linea=False):
		if (nombre!=None):
			# (old_nombre,old_minX,old_maxX,old_minY,old_maxY,old_linea)=self.valor("test")
			old_minX=self.valor("test")[1]
			old_maxX=self.valor("test")[2]
			old_minY=self.valor("test")[3]
			old_maxY=self.valor("test")[4]
			if (minX==None):
				minX=old_minX
			if (maxX==None):
				maxX=old_maxX
			if (minY==None):
				minY=old_minY
			if (maxY==None):
				maxY=old_maxY
			self._test=(nombre,minX,maxX,minY,maxY,linea)
		else:
			return self._test
	# }}}1
	# Evalua una operacion (soporta NA) {{{1
	## \internal
	## Dada una operacion evalua su resultado en el entorno de Python
	## \param self -
	## \param a Operando primero
	## \param oper Operador
	## \param b Operando segundo
	## \return Devuelve el resultado de la operacion
	def evalNA(self,a,oper,b):
		if ((a==self.NA()) or (b==self.NA())):
			return self.NA()
		else:
			return eval("a %s b" % (oper))
	# }}}1
	# Funcion para interpolar los datos de dos arrays para hacerlos contener la misma informacion temporal {{{1
	## \internal
	## Interpola los datos de dos arrays para hacerlso contener la misma información temporal. Los valores interpolados son los valores de cada ARRAY.
	## \param self -
	## \param a1 Vector (E,V) 1
	## \param a2 Vector (E,V) 2
	## \retval Devuelve ambos ARRAYs interpolados en una tupla de dos elementos: (ARRAY Primero interpolado, ARRAY Segundo interpolado)
	def interpolar(self,a1,a2):
		# Inicializo los totales
		t1=len(a1)
		t2=len(a2)
		if ((t1<1) or (t2<1)):
			raise IOError,"Error al interpolar: ambos arrays deben contener almenos 1 elemento para comenzar a interpolar"
		# Inicializo los contadores
		c1=0
		c2=0
		# Inicializo los contenedores
		new1=[]
		new2=[]
		# Inicializo los temporales
		#old1=a1[0]
		#old2=a2[0]
		
		# Mientras haya elementos que procesar creo huecos
		while ((c1<t1) or (c2<t2)):
			if ((c1<t1) and ((c2==t2) or (a1[c1][0]<a2[c2][0]))):
				# Cargo el elemento en la lista new1
				new1.append(a1[c1])
				# Cargo un interpolación en la lista new2
				izq=self.NA()
				der=self.NA()
				if (c2-1>=0):
					izq=a2[c2-1]
				if (c2+1<=t2):
					der=a2[c2]
				if ((izq!=self.NA()) and (der!=self.NA())):
					interpolacion=self.eq_recta(izq[0],izq[1],der[0],der[1],a1[c1][0])
					new2.append((a1[c1][0],interpolacion))
				else:
					new2.append((a1[c1][0],self.NA()))
				# Incremento
				c1=c1+1
			elif ((c2<t2) and ((c1==t1) or (a1[c1][0]>a2[c2][0]))):
				# Cargo el elemento en la lista new2
				new2.append(a2[c2])
				# Cargo un interpolación en la lista new1
				izq=self.NA()
				der=self.NA()
				if (c1-1>=0):
					izq=a1[c1-1]
				if (c1+1<=t1):
					der=a1[c1]
				if ((izq!=self.NA()) and (der!=self.NA())):
					interpolacion=self.eq_recta(izq[0],izq[1],der[0],der[1],a2[c2][0])
					new1.append((a2[c2][0],interpolacion))
				else:
					new1.append((a2[c2][0],self.NA()))
				# Incremento
				c2=c2+1
			else:
				# Cargamos ambos elementos en sus listas respectivas
				new1.append(a1[c1])
				new2.append(a2[c2])
				# Ambos son iguales (traspaso el elemento en ambos) (copio)
				c1=c1+1
				c2=c2+1
		
		# Encuentra el recorte del principio de las listas
		total=len(new1)
		counter=0
		NA_ini=-1
		while (counter<total):
			# Encuentra el primer indice de ambas listas que no es 'NA'
			if ((new1[counter]==self.NA()) or (new2[counter]==self.NA())):
				# Anoto el último NA encontrado
				NA_ini=counter-1
				# Salgo del bucle
				break
			# Incrementa el contador
			counter=counter+1
		# Encuentra el recorte del final de las listas
		counter=total-1
		NA_fin=total
		while (counter>=0):
			# Encuentra el primer indice de ambas listas que no es 'NA'
			if ((new1[counter]==self.NA()) or (new2[counter]==self.NA())):
				# Anoto el último NA encontrado
				NA_fin=counter+1
				# Salgo del bucle
				break
			# Incrementa el contador
			counter=counter-1
		# Recorta ambas listas
		result1=new1[NA_ini+1:NA_fin]
		result2=new2[NA_ini+1:NA_fin]
		# Devuelvo el resultado
		return (result1,result2)
	# }}}1
	# Operadora {{{1
	## \internal
	## Opera vectores (EPOCH,VALOR)
	## \param self -
	## \param acumulador Resultados a computar
	## \param operador Operador a usar
	## \param senal_datos Contiene la nueva senal
	## \param senalnombre Nombre de la linea que se esta operando
	## \param solo_operaciones_digitales Es un valor verdadero o falso
	## \param senal_nombre Nombre de la senal que opera sobre la linea
	def operadora(self,acumulador,operador,senal_datos,senalnombre,solo_operaciones_digitales,senal_nombre):
		# Interpolar datos para igualar contenidos temporales
		(acumulador,senal_datos)=self.interpolar(acumulador,senal_datos)
		if (acumulador==None):
			# No hay datos para procesar
			self.debug("    El acumulador se ha quedado vacio tras interpolarlo con la senal '%s'" % (senalnombre))
			# Vacio el acumulador
			acumulador=None
		else:
			# Operar con lo que tenemos
			acumulador=map(lambda x,y:(x[0],self.evalNA(x[1],operador,y[1])),acumulador,senal_datos)
			# Control de errores
			if (acumulador==None):
				# No hay datos para procesar
				self.debug("    El acumulador se ha quedado vacio tras operarlo con la senal '%s'" % (senalnombre))
				# Vacio el acumulador
				acumulador=None
			else:
				# Control de operaciones digitales
				if (solo_operaciones_digitales):
					self.digitalizar(acumulador,senalnombre,senal_nombre)
		return acumulador
	# }}}1
	# Calculadora de senales {{{1
	## \internal
	## Calculadora de senales
	## \param self -
	## \param senalnombre Contiene la cadena con el comando a ejecutar
	## \param solo_operaciones_digitales Indica si solo se van a realizar operaciones digitales (True/False)
	## \param ini_epoch Fecha de inicio
	## \param fin_epoch Fecha de fin
	## \param casillas Numero de casillas
	## \param minX X minimo
	## \param maxX X maximo
	## \param desviacion Desviacion a usar
	## \param minY Y minimo
	## \param maxY Y maximo
	## \param tickY Tamano del tick en Y
	def calculadora_senales(self,senalnombre,solo_operaciones_digitales,ini_epoch,fin_epoch,casillas,minX,maxX,desviacion,minY,maxY,tickY):
		# Limpieza de la formula: doble espacios
		size=len(senalnombre)
		old_size=size+1
		senalnombre_temp=senalnombre
		while (size!=old_size):
			senalnombre_temp=senalnombre_temp.replace("  "," ")
			old_size=size
			size=len(senalnombre_temp)
		if (senalnombre_temp==" "):
			raise IOError,"La ecuacion entregada esta vacia"
		# Limpieza de la formula: cabecera
		if (senalnombre_temp[0]==" "):
			index_ini=1
		else:
			index_ini=0
		# Limpieza de la formula: cola
		if (senalnombre_temp[-1]==" "):
			index_fin=-1
		else:
			index_fin=len(senalnombre_temp)
		# Limpieza de la formula: recoleccion resultados
		senalnombre_clean=senalnombre_temp[index_ini:(index_fin+1)]
		# Particionlo el comando recibido
		senalpiezas=senalnombre_clean.split(" ")
		
		# Comprueba el balanceo de parentesis
		parentesis=0
		for pieza in senalpiezas:
			# Compruebo aperturas y cierres de parentesis
			if (pieza=='('):
				parentesis=parentesis+1
			elif (pieza==')'):
				parentesis=parentesis-1
			# Detecto si hemos cerrado algun parentesis que no fue abierto previamente
			if (parentesis<0):
				raise SyntaxError,"Durante la ecuacion indicada se ha cerrado un parentesis sin haberlo abierto previamente"
		# Compruebo si hemos dejado algun parentesis sin cerrar
		if (parentesis>0):
			raise SyntaxError,"Los parentesis no estan correctamente balanceados, hay %s parentesis que no han sido cerrados." % (parentesis)
		
		# Inicializo señales
		leer_dato=True
		acumulador=[(ini_epoch,0),(fin_epoch,0)]
		operador="+"
		parentesis=Stack()
		
		# Para cada parte del comando
		for pieza in senalpiezas:
			# Inicializo el saltador para que compute siempre
			saltar=False
			# Compruebo parentesis
			if (pieza=='('):
				# Parentesis de apertura encontrado, compruebo que estamos para leer un dato (equiv: leer_dato)
				if (not leer_dato):
					raise SyntaxError,"Parentesis de apertura encontrado cuando se esperaba un operador: no se puede abrir un parentesis despues de un valor, es necesario indicar como va a ser operado dicho valor."
				else:
					# Si el parentesis es correcto, salvo el estado actual
					parentesis.push((acumulador,operador))
					# Inicializo un nuevo estado
					acumulador=[(ini_epoch,0),(fin_epoch,0)]
					operador="+"
					# No computamos nada en este paso
					saltar=True
			elif (pieza==')'):
				# Parentesis de cierre encontrado, compruebo que estamos para leer un operador (equiv: not leer_dato)
				if (leer_dato):
					raise SyntaxError,"Parentesis de cierre encontrado cuando se esperaba un dato: no se puede cerrar un parentesis despues de un operador binario."
				else:
					# Si el parentesis es correcto, recojo el acumulador
					resultado=acumulador
					# Restaura el estado antiguo
					(acumulador,operador)=parentesis.pop()
					# Opero el acumulador con el resultado del parentesis
					acumulador=self.operadora(acumulador,operador,resultado,senalnombre,solo_operaciones_digitales,senal_nombre)
					# Rompo el bucle si el acumulador se ha quedado vacio
					if (acumulador==None):
						break
					# No computamos nada en este paso
					saltar=True
			# Compruebo el siguiente elemento
			if (leer_dato and (not saltar)):
				leer_dato=False
				# Separacion del nombre y la senal (en la pieza)
				senal_split=pieza.split(":")
				operaciones_extras=[]
				if ((len(senal_split))==1):
					# Indico que no hay argumento
					senal_argumentos=None
					pieza=senal_split[0]
				else:
					# Obtengo el argumento
					senal_argumento=pieza[len(senal_split[0])+1:]
					pieza=senal_split[0]
					# Control de parentesis
					if ((senal_argumento[0]!='(') or (senal_argumento[-1]!=')')):
						raise IOError,"El argumento '%s' entregado con la senal '%s' no esta entre parentesis" % (senal_argumento,pieza)
					# Elimino los parentesis
					senal_argumento=senal_argumento[1:-1]
					# Particiono el argumento
					senal_resplit=senal_argumento.split(",")
					# Control de la ','
					if (len(senal_resplit)==1):
						raise IOError,"El argumento '%s' entregado con la senal '%s' no tiene coma" % (senal_argumento,pieza)
					# Cargo los datos del argumento en una terna
					if (len(senal_resplit)==2):
						senal_argumentos=(senal_resplit[0],senal_resplit[1])
					elif (len(senal_resplit)==3):
						senal_argumentos=(senal_resplit[0],senal_resplit[1])
						extraopers=senal_resplit[2]
						operaciones_extras=extraopers.split("_")
					else:
						raise IOError,"El argumento '%s' entregado con la senal '%s' tiene demasiados argumentos" % (senal_argumento,pieza)
				# Obtengo el ID de la senal
				(senal_id,senal_nombre,tt1,tt2,senal_tipo,tt3,tt4,tt5)=self.__tablasenales.search(None,pieza)
				# Compruebo que encontre la senal indicada
				if (senal_id==None):
					# Convierte la senal en un numero
					try:
						senal_datos=float(pieza)
					except Exception,e:
						raise IOError,"No se pudo encontrar la senal '%s', se intento convertir a FLOAT sin exito: %s" % (pieza,e)
					# Opera el acumulador con el nuevo dato
					acumulador=map(lambda x:(x[0],self.evalNA(x[1],operador,senal_datos)),acumulador)
					# Control de errores
					if (acumulador==None):
						# No hay datos para procesar
						self.debug("    El acumulador se ha quedado vacio tras operarlo con la senal '%s'" % (senalnombre))
						# Vacio el acumulador
						acumulador=None
						# Salgo del bucle
						break
					else:
						# Control de operaciones digitales
						if (solo_operaciones_digitales):
							self.digitalizar(acumulador,senalnombre,senal_nombre)
				else:
					# Obtengo un lector
					lector=self.valor("lector")
					# Transformo los argumentos
					if (senal_argumentos!=None):
						# Obtengo los nuevos limites, minimo
						if (senal_argumentos[0]=='?'):
							minX_new=minX
						else:
							entregar=separar_operadores(senal_argumentos[0])
							# senal_tmp=senal_argumentos[0]
							minX_new=calculadora(entregar,self.viaje_temporal())
						# Maximo
						if (senal_argumentos[1]=='?'):
							maxX_new=maxX
						else:
							# senal_tmp=senal_argumentos[0]
							entregar=separar_operadores(senal_argumentos[1])
							maxX_new=calculadora("%s" % (entregar),self.viaje_temporal())
						# Solicito al lector que precargue los datos de la senal que voy a leer
						lector.load((senal_id,senal_tipo,minX_new,maxX_new))
					else:
						# Solicito al lector que precargue los datos de la senal que voy a leer
						lector.load((senal_id,senal_tipo,minX,maxX))
						# Operaciones extras
						operaciones_extras=[]
					# Obtengo los datos de la senal
					senal_datos_dirty=lector.getall()
					if (senal_datos_dirty!=None):
						# Intercambio los None por NA
						senal_datos=map(lambda x:(x[0],self.None2NA(x[1])),senal_datos_dirty)
					else:
						senal_datos=None
					# Control de que hay datos que procesar
					if (senal_datos==None):
						# No hay datos para procesar
						self.debug("    No se han encontrado datos en el lector para la senal '%s' entregada como '%s'" % (pieza,senalnombre))
						# Vacio el acumulador
						acumulador=None
						# Salgo del bucle
						break
					else:
						# Compruebo si tengo que autoescalar antes de interpolar
						for operacion in operaciones_extras:
							if (operacion=="autoescalar"):
								senal_datos=escalarX(senal_datos,minX,maxX+desviacion)
						# Opero el resultado
						acumulador=self.operadora(acumulador,operador,senal_datos,senalnombre,solo_operaciones_digitales,senal_nombre)
						# Rompo el bucle si el acumulador se ha quedado vacio
						if (acumulador==None):
							break
			elif ((not leer_dato) and (not saltar)):
				leer_dato=True
				# Recoje un operador
				operador_split=pieza.split(":")
				operador=operador_split[0]
				if (len(operador_split)>1):
					operador_argumentos=operador_split[1]
				else:
					operador_argumentos=None
				# Control de operadores unarios
				if (operador=="not"):
					# Lee otro operador
					leer_dato=False
					# Acumula el resulado
					acumulador=logicoNot(acumulador)
				elif (operador=="sqrt"):
					# Lee otro operador
					leer_dato=False
					# Acumula el resulado
					acumulador=sqrtEV(acumulador)
					# Control de operaciones digitales
					if (solo_operaciones_digitales):
						self.digitalizar(acumulador,senalnombre,senal_nombre)
				elif (operador=="axis"):
					# Lee otro operador
					leer_dato=False
					# Busco el eje a procesar
					encontrado=None
					for axis in self.__axis:
						if (str(axis[0])==operador_argumentos):
							encontrado=axis
							break
					if (encontrado==None):
						raise IOError,"No se ha encontrado el eje %s" % (operador_argumentos)
					else:
						# Recojo los datos
						if (axis[1]=="der"):
							# Recoje los datos del eje
							# (aid,aorilla,atexto,aposicion,acolor,atick,atransformador)=axis
							atransformador=axis[6]
							# Carga el transformador
							(aminY,amaxY,astep,alabelmin,alabelmax,alabelstep)=self._transformar(atransformador,minY,maxY,tickY)
							# Extraigo los datos
							axis_minY=aminY
							axis_maxY=amaxY
							proporcion=float(axis_maxY-axis_minY)/float(alabelmax-alabelmin)
							# Traslacion del eje Y  al eje 0
							operador="-"
							valor=alabelmin
							acumulador=map(lambda x:(x[0],self.evalNA(x[1],operador,valor)),acumulador)
							# Escalado del ejeY
							operador="*"
							valor=proporcion
							acumulador=map(lambda x:(x[0],self.evalNA(x[1],operador,valor)),acumulador)
							# Traslacion del eje Y a la posición final
							operador="+"
							valor=axis_minY
							acumulador=map(lambda x:(x[0],self.evalNA(x[1],operador,valor)),acumulador)
							# Control de operaciones digitales
							if (solo_operaciones_digitales):
								self.digitalizar(acumulador,senalnombre,senal_nombre)
						elif (axis[1]=="sup"):
							# Recoje los datos del eje
							(aid,aorilla,atexto,aposicion,acolor,atick,aenlace)=axis
							# Carga el transformador
							(afechaini,afechafin,aini,acrece,astep)=aenlace
							# Creo los limites
							limini=calculadora(afechaini,self.viaje_temporal())
							limfin=calculadora(afechafin,self.viaje_temporal())
							#limstep=self.calculadora(astep,self.viaje_temporal())
							# Extraigo los datos
							axis_minX=limini
							axis_maxX=limfin
							# Escalado en el ejeX
							acumulador=self.escalarX(acumulador,axis_minX,axis_maxX)
						else:
							raise IOError,"El eje al que se ha enlazado no es ni superior, ni derecho, no se puede realizar el escalado"
				elif (operador=="^"):
					# Compatibilidad con otros lenguajes
					operador="**"
				elif (operador=="autoescalar"):
					# Lee otro operador
					leer_dato=False
					# Escala la senal al rango disponible de ventana (escalado en el eje X con analisis de datos)
					acumulador=self.escalarX(acumulador,minX,maxX+desviacion)
				elif (operador=="notNeg"):
					# Lee otro operador
					leer_dato=False
					# Anula los valores negativos haciendolos cero
					acumulador=self.notNeg(acumulador)
				elif (operador=="acumular"):
					# Lee otro operador
					leer_dato=False
					# Genera un vector de valores acumulador
					acumulador=self.acumular(acumulador)
				elif (operador=="desacumular"):
					# Lee otro operador
					leer_dato=False
					# Genera un vector de valores acumulador
					acumulador=self.desacumular(acumulador)
				elif (operador=="resetNull"):
					# Lee otro operador
					leer_dato=False
					# Genera un vector de valores acumulador
					acumulador=self.resetNull(acumulador,operador_argumentos)
				elif ((operador=="max") or (operador=="min") or (operador=="med")):
					# Lee otro operador
					leer_dato=False
					# Maximos, minimos y media, reviso cuantos puntos se desean
					if (operador_argumentos!=None):
						# Tantas particiones como se hayan indicado
						num_particiones=int(operador_argumentos)
					else:
						# Tantas particiones como puntos haya para poner datos
						num_particiones=casillas
					# Particiono el array
					particiones=particionar(acumulador,num_particiones)
					# Proceso las particiones generadas
					acumulador=[]
					for lista in particiones:
						# Calculo el resultado
						if (operador=="max"):
							elemento=self.maximo(lista)
						elif (operador=="min"):
							elemento=self.minimo(lista)
						elif (operador=="med"):
							elemento=self.media(lista)
						
						# Anado el elemento al acumulador
						if (elemento!=None):
							acumulador.append(elemento)
				# Operadores python permitidos (registra aqui cualquier operador Python que se desee usar y que este disponible en el lenguaje)
				elif ((operador=="+")
					or (operador=="-")
					or (operador=="*")
					or (operador=="/")
					or (operador=="%")
					or (operador=="**")
					or (operador=="and")
					or (operador=="or")
					or (operador=="==")
					or (operador=="!=")
					or (operador==">")
					or (operador=="<")
					or (operador==">=")
					or (operador=="<=")
					):
					pass
				# Operadores desconocidos o no registrados
				else:
					raise IOError,"Operador %s no encontrado" % (operador)
		return acumulador
# }}}1
	# Funcion para calcular el maximo de un array con ternas (epoch,valor) {{{1
	## \internal
	## Funcion para calcular el maximo de un ARRAY EV
	## \param self -
	## \param vector ARRAY con los datos en formato EV
	## \return Devuelve el elemento EV maximo del ARRAY EV
	def maximo(self,vector):
		maximo=None
		for elemento in vector:
			if ((elemento[1]!=None) and (elemento[1]!=self.NA()) and ((maximo==None) or (elemento[1]>maximo[1]))):
				maximo=elemento
		return maximo
	# }}}1
	# Funcion para calcular el minimo de un array con ternas (epoch,valor) {{{1
	## \internal
	## Funcion para calcular el minimo de un ARRAY EV
	## \param self -
	## \param vector ARRAY con los datos en formato EV
	## \return Devuelve el elemento EV minimo del ARRAY EV
	def minimo(self,vector):
		minimo=None
		for elemento in vector:
			if ((elemento[1]!=None) and (elemento[1]!=self.NA()) and ((minimo==None) or (elemento[1]<minimo[1]))):
				minimo=elemento
		return minimo
	# }}}1
	# Funcion para calcular la media de un array con ternas (epoch,valor) (Elimina los segundos elementos repetidos) {{{1
	## \internal
	## Funcion para calcular la media de un ARRAY EV
	## \param self -
	## \param vector ARRAY con los datos en formato EV
	## \return Devuelve un elemento EV que representa en valor a la media del ARRAY EV entregado
	## \warning Esta funcion no usa los segundos elementos repetidos, es decir, que si ocurre en el array 2 valores distintos en el mismo instante, esta funcion no computara en la media el segundo valor, sea cual sea.
	def media(self,vector):
		if (len(vector)<1):
			media=None
		elif (len(vector)==1):
			media=vector[0]
		else:
			# Inicializo los valores
			media=0
			suma=0
			total=0
			ini=None
			# Para cada elemento del vector
			for elemento in vector:
				# Si no hay elemento inicial
				if (ini==None):
					# Cargo el elemento inicial
					ini=elemento
				else:
					# Busco el elemento final porque el tiempo ha cambiado (buscamos pares)
					if (elemento[0]!=ini[0]):
						# Cargo el elemento final
						fin=elemento
						# Si el valor no es None o NA
						if ((ini[1]!=None) and (ini[1]!=self.NA())):
							# Cargo el valor al sumatorio
							suma=suma+(fin[0]-ini[0])*ini[1]
							# Cargo el total de elementos procesados al total
							total=total+(fin[0]-ini[0])
					# Cargo un nuevo elemento inicial
					ini=elemento
			# Devuelvo la media (total del sumatorio / numero de elementos)
			if (total==0):
				media=(elemento[0],0)
			else:
				media=(elemento[0],float(suma)/float(total))
		# Devuelvo la media obtenida
		return media
	# }}}1
	# Funcion que realiza un Not del vector {{{1
	## \internal
	## Funcion para realiza un Not del vector
	## \param self -
	## \param vector ARRAY con los datos en formato EV
	## \return Devuelve un array de elementos EV cuyo valores han sido transformados por la funcion Not
	def logicoNot(self,vector):
		new_lista=[]
		# Para cada elemento del vector
		for elemento in vector:
			# Cargo el elemento a la nueva lista
			if (elemento[1]==None):
				new_lista.append(elemento)
			else:
				new_lista.append((elemento[0],not elemento[1]))
		# Devuelvo la nueva lista
		return new_lista
	# }}}1
	# Funcion que transforma los negativos de una lista en cero {{{1
	## \internal
	## Funcion para transforma los negativos de una lista en cero)
	## \param self -
	## \param vector ARRAY con los datos en formato EV
	## \return Devuelve un array de elementos EV cuyo valores negativos fueron intercambiados con cero
	def notNeg(self,vector):
		new_lista=[]
		# Para cada elemento del vector
		for elemento in vector:
			# Salto los elementos que valgan 0
			if (elemento[1]<0):
				elemento=(elemento[0],0)
			# Cargo el elemento a la nueva lista
			new_lista.append(elemento)
		# Devuelvo la nueva lista
		return new_lista
	# }}}1
	# Funcion que acumula los valores de una lista {{{1
	## \internal
	## Funcion para acumula los valores de una lista
	## \param self -
	## \param vector ARRAY con los datos en formato EV
	## \return Devuelve un array de elementos EV cuyo valores han sido acumulados
	def acumular(self,vector):
		new_lista=[]
		# Para cada elemento del vector
		old=(self.NA(),self.NA())
		for elemento in vector:
			if (old[1]==self.NA()):
				# Actualizo old
				old=elemento
			else:
				# Recalculo el valor si elemento no es nulo
				if (elemento[1]!=self.NA()):
					# Recalculo el valor
					elemento=(elemento[0],old[1]+elemento[1]*(elemento[0]-old[0]))
					# Actualizo old
					old=elemento
			# Cargo el elemento a la nueva lista
			new_lista.append(elemento)
		# Devuelvo la nueva lista
		return new_lista
	# }}}1
	# Funcion que desacumula los valores de una lista {{{1
	## \internal
	## Funcion para desacumula los valores de una lista
	## \param self -
	## \param vector ARRAY con los datos en formato EV
	## \return Devuelve un array de elementos EV cuyo valores han sido acumulados
	def desacumular(self,vector):
		new_lista=[]
		# Para cada elemento del vector
		old=(self.NA(),self.NA())
		for elemento in vector:
			if (old[1]==self.NA()):
				old=elemento
				new_lista.append((elemento[0],0.0))
			elif ((old[1]!=elemento[1]) or (old==vector[0]) or (elemento==vector[-1])):
				if ((elemento[1]==self.NA()) or (old[1]==self.NA())):
					# Actulizo old
					old=elemento
				else:
					# Recalculo el valor
					value=(elemento[1]-old[1])/(elemento[0]-old[0])
					# Actulizo old
					old=elemento
					# Actualizo el elemento
					elemento=(elemento[0],value)
				# Cargo el elemento a la nueva lista
				new_lista.append(elemento)
		# Devuelvo la nueva lista
		return new_lista
	# }}}1
	# Funcion que intercambia los valores nulos por otro valor {{{1
	## \internal
	## Funcion que intercambia los valores nulos por otro valor
	## \param self -
	## \param vector ARRAY con los datos en formato EV
	## \param setValue Nuevo valor que poner
	## \return Devuelve un array de elementos EV cuyo valores nulos han sido intercambiados por otro valor
	def resetNull(self,vector,setValue):
		# Get new Value
		if (setValue==None):
			newValue=0
		else:
			newValue=float(setValue)
		# Para cada elemento del vector
		new_lista=[]
		for elemento in vector:
			# Obtengo el valor
			valor=elemento[1]
			# Miro si es nulo
			if (valor==self.NA()):
				# Reasigno su valor
				valor=newValue
			# Lo paso a la nueva lista
			new_lista.append((elemento[0],valor))
		# Devuelvo la nueva lista
		return new_lista
	# }}}1

	# Anade una linea a la grafica {{{1
	## Inserta una linea en la grafica:
	##
	## \param self -
	## \param nombre Nombre de la linea (debe ser unico)
	## \param senal Cadena de texto con las senales a obtener y como operarlas. Puede contener el siguiente formato: "AAA BBB CCC ...", donde cada elemento es procesado en la calculadora() \n
	## - AAA puede ser un operador o un valor, los valores pueden ser valores enteros, valores reales o señales de la tabla de señales. \n
	## - Las señales soportan argumentos (para seleccionar por ejemplo el rango a procesar), si no se indica se usa el rango indicado en sizex. \n
	## - Los operadores pueden ser cualquier procesable por python, sqrt, ^, med, max, min, axis. Los 4 últimos además soportan argumentos propios que se escriben separando el operador del argumento con el caracter ":" (Ejemplo: "med:4" obtiene 4 medias), en el caso de med (media), max (maximos) y min (minimos) se indica el número de elementos a obtener de la gráfica (cuantas medias, máximos o minimos), en el caso de axis se indica el id del eje al que enlazar el escalado que deseamos realizar. \n
	## - Es posible sumar y restar unas señales con otras, o multiplicarlas por un valor. Si dos señales tienen elementos distintos en el tiempo, los elementos que no coinciden son interpolados. \n
	## \n
	## Ejemplo: \n
	## "C1_Torredelmar:(22/08/2006_07:15:00,23/08/2006_07:15:00) * 5 med:10" \n
	## Coje la señal C1_Torredelmar, saca los datos entre el 22/08/2006 07:15 y 23/08/2006 07:15, multiplicala por 5, y muestrame 10 medias del resultado
	## \n
	## \param senaltipo 'a' analogica o 'd' digital
	## \param linea Indica cómo se va a pintar la línea: escalera, lineal, maximos, minimos, medias y evento (puede contener parametros, para ello se separan con ":")
	## \param color Representa el color de la línea, en los eventos debe ser un listado de los colores que van a ser mostrados en este evento. Su formato es una tupla de 3 elementos (color de las letras,lista de colores,color para valores fuera del rango). Ver parametro \c \b tipo. \n
	## \n
	## \param grosor Representa el grosor de la línea
	## \param tipo Indica el tipo de línea, generalmente solida, a puntos, rayas y puntos, etc...los eventos, media, máximos y mínimos tienen su propio formato explicado abajo \n
	## - Explicacion MAXIMOS-MINIMOS: representa (tipo1,tipo2,ponertexto) donde tipo1 y tipo2, son tipos de puntos para representar los MAXIMO-MINIMOS, el tipo1 son los que no sobrepasan los límites de la gráfica (MAXIMOS-MINIMOS normales), el tipo2 son los que sobrepasan los límites de la gráfica (mayores que el máximo posible de la gráfica). Poner texto significa que hay que pintar el texto en cada punto (Booleano: True-False). \n
	## \n
	## - Explicacion MEDIA: representa (tipo,ponertexto), básicamente es el tipo de cada punto de la media y si hay que poner o no el texto (Booleano: True-False). El filtro indica si los valores dados en el rango están ajustados o no. Cuando un valor está ajustado significa que no existe color para los valores de enmedio, es decir, que los valores dados sólo son pintados cuando el valor está exactamente en el rango con un color asignado, en caso de que el valor esté entre 2 valores del rango, no dispondrá de color para ser pintado. \n
	## \n
	## Ejemplo: \n
	## g.addline("Evento 3","C1_Torredelmar:(22/08/2006_07:15:00,23/08/2006_07:15:00) * 5 med:10","a","med",azul_claro,0.5,(circuloC,False)) \n
	## \n
	## - Explicacion EVENTOS: representa el rango de colores y el rango de valores \n
	## \n
	##  La tupla de color est formada por 3 elementos: ("Color del texto",[lista_colores_como_se_ve_a_continuacion],"Color en caso de que falle") \n
	##  El color None (sin comillas) es el color transparente, o mejor dicho, la ausencia de color.\n
	## \n
	##  - \b Rangos \b ajustados: La señal puede usar el filtro a True para indicar que los rangos de colores y valores están ajustados, lo que significa que se mostrará el color X1 cuando la gráfica cruce el valor Y1, y el color X2 cuando la gráfica cruze el valor X2, no cuando el valor de la gráfica está entre Y1 e Y2. En el cajo de rangos ajustados es obligatorio que la lista de colores y la lista de valores tengan la misma longitud para enlazar cada valor con su color. \n
	## \n
	## Ejemplo ajustado: \n
	## valores=[1,2,3] \n
	## colores=[rojo,azul,verde] \n
	## Cuando la gráfica vale 1 y sólo 1, se pinta rojo \n
	## Cuando la gráfica vale 2 y sólo 2, se pinta azul \n
	## Cuando la gráfica vale 3 y sólo 3, se pinta verde \n
	## \n
	##  - \b Rango \b de \b valores: (su formato es una lista de números) contiene un listado de valores que será usado para mostrar los colores, hay 2 tipos: \n
	## \n
	##   - a) Cuando el listado de valores es de longitud 2, almenos es necesario un color en la lista de colores. Lo que se hace es particionar el espacio entre el valor 1 y el valor 2 en tantos huecos como colores haya, y cuando la gráfica esté pasando por un hueco, se pinta el evento con el color de ese hueco. \n
	## \n
	## Ejemplo a: \n
	## valores=[0,32] \n
	## colores=[rojo,azul,verde,amarillo] \n
	## Cuando la gráfica está entre 0 y 8, se pinta rojo \n
	## Cuando la gráfica está entre 8 y 16, se pinta azul \n
	## Cuando la gráfica está entre 16 y 24, se pinta verde \n
	## Cuando la gráfica está entre 24 y 32, se pinta amarillo \n
	## \n
	##   - b) Cuando el listado de valores es de longitud mayor que 2, es obligatorio que el listado de colores sea de longitud de la lista de valores menos 1, en este caso se la lista de valores representa los huecos y el primer color se asignará al primer hueco (correspondiente al hueco dejado entre el valor 1 y el valor 2).\n
	## \n
	## Ejemplo b: \n
	## valores=[0,5,10,15,32] \n
	## colores=[rojo,azul,verde,amarillo] \n
	## Cuando la grafica está entre 0 y 5, se pinta rojo \n
	## Cuando la grafica está entre 5 y 10, se pinta azul \n
	## Cuando la grafica está entre 10 y 15, se pinta verde \n
	## Cuando la grafica está entre 15 y 32, se pinta amarillo

	## \param filtro Generalmente no se usa, solo en la media. Mirar parametro \c \b tipo. \n
	## \return Devuelve True si todo ha ido bien
	## \exception IOError Si el nombre ya fue insertado previamente
	## \sa addaxis()
	def addline(self,nombre,senal,senaltipo,linea,color,grosor,tipo,filtro=None):
		# Comprueba que el id y el alfanum no existen en la tabla
		for temp_senal in self.__tablalineas:
			if (temp_senal[0]==nombre):
				error="ERROR GRAFICAS: El NOMBRE %s ya existe en la configuacion de graficas" % (nombre)
				# Termino porque fue mal
				raise IOError,error
		# Descompongo el tipo de linea
		linea_split=linea.split(":")
		linea_tipo=linea_split[0]
		if (len(linea_split)==1):
			linea_argumento=None
		else:
			linea_argumento=linea_split[1]
		# Anado la senal a la tabla
		self.__tablalineas.append((nombre,(senal,senaltipo),(linea_tipo,linea_argumento,color,grosor,tipo),filtro,False))
		# Devuelvo que todo fue bien
		return True
	# }}}1
	# Anade un texto a la grafica {{{1
	## Inserta un texto
	## \param self -
	## \param nombre Nombre de la senal
	## \param texto Texto a mostrar
	## \param x Posicion en eje X
	## \param y Posicion en eje Y
	## \param colorfg Color de la fuente
	## \param colorbg Color del fondo
	## \param size Tamano de la fuente
	def addtext(self,nombre,texto,x,y,colorfg,colorbg,size):
		# Comprueba que el id y el alfanum no existen en la tabla
		for temp_senal in self.__tablalineas:
			if (temp_senal[0]==nombre):
				error="ERROR R: El TEXTO %s ya existe en la configuacion de graficas" % (nombre)
				# Termino porque fue mal
				raise IOError,error
		# Añado la senal a la tabla
		self.__tablalineas.append((nombre,(texto,'t'),("texto",(x,y),(colorfg,colorbg),size,None),None,False))
		# Devuelvo que todo fue bien
		return True
	# }}}1
	# Anade un eje {{{1
	## Inserta un eje en la grafica
	## \param self -
	## \param id Se ponga un entero, un decimal o una cadena, es tranformado a cadena por la funcion str() de Python, y es un dato de referencia para poder hacer que las lineas puedan autoescalarse con las propiedades de un eje dado. Si id tiene un texto entre el caracter ":" (Ejemplo: ":texto:") se interpretara como un eje especial. Las opciones son ":texto:" (muestra el texto solamente), ":escala:" (muestra la escala solamente), ":escala+texto:" o ":texto+escala:" (muestra el texto y la escala). Si el id es "0" se interpretara como ":texto+escala:"
	## \param orilla Puede ser: "sup", "inf", "der", "izq", cuando se indica añadir una orilla "izq" o "inf", se usará sólamente su texto para sustituir al texto de ese eje, el resto de propiedades no serán usadas, además si se indican varios inferiores (inf) o izquierdos (izq), se usará sólamente el último indicado de los inferiores y el último de izquierdos. Por lo tanto el uso real del graficador y el transformador se aplica a las orilla derecha (der) y superior (sup).
	## \param texto Texto que poner en el eje
	## \param graficador Contiene datos para pintar: \n
	## - a) (posicion de la leyenda) -> (1.3) \n
	## - b) (posicion de la leyenda, color) -> (1.3,rojo) \n
	## - c) (posicion de la leyenda, color, tamaño del tick) -> (1.3,rojo,-0.008) \n
	## Si el tick es >0 sale por la derecha del eje, tick <0 sale por la izq del eje
	## \param transformador Contiene datos de transformación del eje: \n
	## - a) (minimo real, maximo real, minimo virtual, maximo virtual, tamaño de los pasos) -> (50,800,1,22,1) \n
	## Se escala el resultado de la función poniendo en 50 lo que en la función vale 1 y en 800 lo que en la función vale 22, y se pintan los ticks en pasos de 1. Por supuesto si un valor está entre 1 y 22, se escala a su proporcion exacta en la escala entre 50 y 800 \n
	## \n
	## - b) ("texto de transformación" -> ("* 1.3 + 100") ¡¡¡[NO SOPORTADO POR "sup"]!!! \n
	## Se coje el eje izquierdo y se multiplica la escala por 1.3 y se le suma 100, los valores del eje son los resultantes de la transformación \n
	## \n
	## - c) ("texto de transformación",minimo virtual,maximo virtual) -> ("* 1.3 + 100", 100, 300) ¡¡¡[NO SOPORTADO POR "sup"]!!! \n
	## Se coje el eje izquierdo y se multiplica la escala por 1.3 y se le suma 100, los valores a pintar comienzan en 100 y terminan en 300, los valores intermedios estarán posicionados según los ticks de la transformación y usando el valor que corresponda en la escala entre 100 y 300.
	## \attention Ejemplos: \n
	## \n
	## g.addaxis(1,"der","Temperatura (ºC)",[1.5,2,-0.008],[50,800,1,22,1]) \n
	## Añade un eje a la derecha llamado "Temperatura (ºC)" que está en la posición x=1.5, usando el color 2 y el tick tiene una longitud de 0.008 hacia la izquierda del eje, los valores y ticks serán pintados entre 50 y 800, poniendo ticks en pasos de 1, contando desde 1 a 22, siendo 1 la posición 50 del eje Y y 22 la posición 800. El id de este eje para referencias es "1". \n
	## \n
	## g.addaxis("Eje","sup",None,[0.3,verde,0.008],["date:22/08/2006_07:15:00","date:23/08/2006_00:15:00","L",1,"15m"]) \n
	## Anade un eje en la parte superior sin nombre!, en la posicion 0.3 por encima del eje superior, usando el color verde, y con los ticks por encima del eje con una longitud de 0.008, la transformacion se realiza haciendo corresponder la fecha 22/07/2006 07:15:00 con el valor "L" (LUNES) y creciendo a pasos de 1 (es decir "M", "X", ...), poniendo una marca cada "15m" (15 minutos), el crecimiento se hace siguiendo esta regla hasta la fecha 23/08/2006 00:15:00am. En resumen: escribira "L" en 22/08/2006 07:15, "M" en 22/08/2006 07:30, "X" en 22/08/2006 07:45,....hasta llegar al 23/08/2006 00:15. Es observable que los valores reales son obtenidos por computacion mediante la calculadora temporal (consultar mas abajo), el tamano de los pasos tambien usa la calculadora temporal (ver mas abajo). El id de este eje para referencias es "Eje".

	## \exception IOError Ocurre cuando el eje que se trata de insertar contiene algun error
	def addaxis(self,id,orilla,texto,graficador=None,transformador=None):
		# Compruebo que la posicion indicada es válida
		if ((orilla!="sup") and (orilla!="inf") and (orilla!="izq") and (orilla!="der")):
			raise TypeError,"La posicion indicada al anadir el eje es invalida, solo 'sup', 'inf', 'izq' o 'der'. Entregada: %s" % (orilla)
		# Compruebo que han llegado todos los parámetros
		if (((orilla=="izq") and (graficador!=None)) or (orilla=="der") or (orilla=="sup")):
			if (len(graficador)<=0):
				raise IOError,"El eje que trata de insertar es erroneo (graficador de tamano 0): (%s,%s,%s)" % (texto,graficador,transformador)
			# Extrae los datos del graficador
			posicion=graficador[0]
			if (len(graficador)>1):
				color=graficador[1]
			else:
				color=2		# Color por defecto
			if (len(graficador)>2):
				tick=graficador[2]
			else:
				tick=0.008	# Tamano de tick por defecto
			if (len(graficador)>3):
				raise IOError,"El eje que trata de insertar tiene un graficador que excede el tamano maximo: (%s,%s,%s,%s)" % (posicion,texto,graficador,transformador)
		else:
			posicion=None
			color=None
			tick=None
			transformador=None
		# Reemplazo el id
		if (str(id)=="0"):
			id=":texto+escala:"
		# Anado el eje a la lista
		self.__axis.append((str(id),orilla,texto,posicion,color,tick,transformador))
	# }}}1

	# Test de existencia de datos {{{1
	## \internal
	## Test de existencia de datos
	## \param self
	## \return Devuelve True si el test es correcto y False si el test devuelve algun error (Ej: no hay datos)
	def dotest(self):
		# Obtengo los datos del test
		(senal_alfaid,minX,maxX,minY,maxY,linea)=self.valor("test")
		# Reviso que hay que realizar el test
		if (senal_alfaid==False):
			# Devuelvo que el test se ha pasado con éxito si no hay que hacerlo
			return True
		# Calculo el traslado de la senal en el tiempo
		minX=calculadora(minX,self.viaje_temporal())
		maxX=calculadora(maxX,self.viaje_temporal())
		# Si nos dan una linea busco el id de la senal
		if (linea==True):
			# Busco el Nombre de la señal en la tabla de lineas insertadas
			encontrado=False
			for senal in self.__tablalineas:
				# Extrae los datos de la senal
				# (busca_nombre,(busca_senal,busca_senaltipo),tmp1,tmp2,tmp3)=senal
				busca_nombre=senal[0]
				busca_senal=senal[1][0]
				# Comprueba si es la que estamos buscando
				if (busca_nombre==senal_alfaid):
					# Si lo extrae el nombreID
					senal_alfaid=busca_senal
					# Anota que ha sido encontrado el resultado
					encontrado=True
					# Sale del bucle
					break
			# En caso de que no se haya encontrado ninguna
			if (not encontrado):
				# Se devuelve que no hay datos
				return False
		# Obtengo los datos de la senal
		tmpidsenal=self.__tablasenales.search(None,senal_alfaid)
		(senal_id,senal_nombre,tt1,tt2,senal_tipo,tt3,tt4,tt5)=tmpidsenal
		# Obtengo el lector
		lector=self.valor("lector")
		# Guardo el nombre de la variable de control
		lector.load((senal_id,senal_tipo,minX,maxX))
		# Obtengo los datos de la senal
		senal_datos=lector.getall()
		
		if (senal_datos!=None):
			# Potencialmente hay resultados (comprobemos la viabilidad entre minY y maxY)
			encontrado=False
			# Busco en el resultados si hay algún valor que cumple los límites minY<=X<=maxY
			for elemento in senal_datos:
				if ((elemento[1]>=minY) and (elemento[1]<=maxY)):
					# Se encontró un elemento en el resultado que cumple los limites
					encontrado=True
					# Termino el bucle
					break
			# Devuelvo el resultado del test
			return encontrado
		else:
			# No hay resultados
			return False
	# }}}1
	# Transformador (calcula la matriz de transformacion de un array transformador) {{{1
	## \internal
	## Calcula la matriz de transformacion de un array transformador
	## \param self -
	## \param transformador Cadena o tupla con informacion sobre la transformacion a realizar
	## \param minY Posicion minima en el eje Y
	## \param maxY Posicion maximo en el eje Y
	## \param tickXY Distancia entre valores consecutivos del eje Y
	## \return Devuelve una tupla (minY,maxY,step,labelmin,labelmax,labelstep): (posicion original minima en el eje Y, posicion original maxima en el eje Y, tamano original de cada paso en el eje Y, posicion virtual minima en el eje Y, posicion virtual maxima en el eje Y, tamano virtual de cada paso en el eje Y)
	## \exception IOError Ocurre cuando el transformador es incorrecto
	def _transformar(self,transformador,minY,maxY,tickY):
		if (len(transformador)==1):
			# Transformador de texto
			aminY=minY
			amaxY=maxY
			alabelmin=calculadora("%s %s" % (minY,transformador[0]),self.viaje_temporal())
			alabelmax=calculadora("%s %s" % (maxY,transformador[0]),self.viaje_temporal())
			alabelstep=calculadora("%s %s" % (tickY,transformador[0]),self.viaje_temporal())
		elif (len(transformador)==3):
			# Extraigo los datos
			transfor=transformador[0]
			alabelmin=transformador[1]
			alabelmax=transformador[2]
			# Transformador de texto con maximo y minimo
			aminY=minY
			amaxY=maxY
			alabelstep=calculadora("%s %s" % (tickY,transfor),self.viaje_temporal())
		elif (len(transformador)==5):
			# Transformador de matriz
			aminY=transformador[0]
			amaxY=transformador[1]
			alabelmin=transformador[2]
			alabelmax=transformador[3]
			alabelstep=transformador[4]
		else:
			# Transformador incorrecto
			raise IOError,"El transformador es incorrecto: %s" % (transformador)
		# Calcula los pasos que tiene que dar en la función real
		astep=float(amaxY-aminY)/(float(alabelmax-alabelmin)/float(alabelstep))
		# Devuelvela tupla con el resutado
		return ((aminY,amaxY,astep,alabelmin,alabelmax,alabelstep))
	# }}}1
	# Digitaliza los datos de una senal {{{1
	## \internal
	## Digitaliza los datos de una senal
	## \param self -
	## \param datos ARRAY con los datos a procesar
	## \param linea Nombre de la linea que se esta operando
	## \param senal Nombre de la senal que opera sobre la linea
	## \return Devuelve un ARRAY identico a datos pero digitalizado
	def digitalizar(self,datos,linea,senal):
		# Inicializo el array nuevo
		nueva=[]
		primeraviso=True
		# Para todos los datos
		for dato in datos:
			newdato=dato[1]
			if ((dato[1]!=0) and (dato[1]!=1) and (dato[1]!=self.NA())):
				# Debug
				if (primeraviso):
					if (senal==None):
						self.debug("       > Digitalizador: operacion de valor analogico sobre la digital '%s'" % (linea))
					else:
						self.debug("       > Digitalizador: operacion de datos analogicos de '%s' en la digital '%s'" % (senal,linea))
					primeraviso=False
				if ((dato[1]!=0) and (dato[1]!=self.NA())):
					newdato=1.0
			# Digitaliza el dato
			nueva.append((dato[0],newdato))
		# Devuelve el array digitalizado
		return nueva
	# }}}1
	# Eventualizar {{{1
	## \internal
	## Eventualiza los datos dados
	## \param self -
	## \param datos ARRAY con los datos a eventualizar
	## \param rango Lista de valores con lo que operar los colores
	## \param colores Lista de colores con los que proceder
	## \param colorNegativo Color a usar cuando no hay ningun color que se ajuste a la decision del rango
	## \param grosor Grosor de cada linea de los eventos
	## \param ajustado Indica si se usa ajuste o no en la asignacion de valores y colores (True/False)
	## \param linea Tipo de linea a usar para pintar
	## \param posicion Posicion en la que comenzar a pintar los eventos
	## \return Devuelve una lista con los eventos a pintar
	## \exception IOError Ocurre cuando se produce algun error al eventualizar los datos
	def eventualizar(self,datos,rango,colores,colorNegativo,grosor,ajustado,linea,posicion):
		# Inicializo el resultado
		result=[]
		# Si tengo datos para procesar, computo los resultados
		if (len(datos)>0):
			# Control de parametros {{{2
			if (( type(rango)!=type([]) ) or ( type(colores)!=type([]) )):
				raise IOError,"ERROR: El eventualizador ha recibido en la linea %s un rango/colores que no están dentro de una lista" % (linea)
			# }}}2
			# Genero los pares rango-color {{{2
			rancol=[]
			if ((ajustado) and (len(rango)>0) and (len(rango)==len(colores))):
				# Asigno los pares uno a uno (cada mochuelo a su olivo) -> (0,1,1) (A,B,C)
				for i in range(0,len(rango)):
					rancol.append((rango[i],rango[i],colores[i]))
			elif ((not ajustado) and (len(rango)==2) and (len(colores)>0)):
				# Particiono el rango con el espacio de colores -> (0,0.5,1) (A,B)
				ini=float(rango[0])
				fin=float(rango[1])
				total=float(len(colores))
				salto=(fin-ini)/total
				i=ini
				c=0
				while (i+salto<=fin):
					rancol.append((i,i+salto,colores[c]))
					i=i+salto
					c=c+1
			elif ((not ajustado) and (len(rango)>2) and ((len(rango))==(len(colores)+1))):
				# Asigno cada color al rango correspondiente (No hay que transformar la lista de rangos)
				for i in range(0,len(colores)):
					rancol.append((rango[i],rango[i+1],colores[i]))
			else:
				IOError,"ERROR: Las listas entregadas al eventualizador en la linea %s no coinciden con ninguno de los formatos permitidos" % (linea)
			# }}}2
			# Creo la lista de eventos {{{2
			tipo=1		# Siempre sera una linea (1)
			result=[]
			color=None
			# oldtipo=tipo
			# oldgrosor=grosor
			first=True
			ini=datos[0]
			fin=None
			for dato in datos:
				if (first):
					first=False
				else:
					if (color!=None):
						# Uso el color encontrado
						result.append(([(ini[0],posicion),(dato[0],posicion)],tipo,grosor,color))
					else:
						# En caso de que no haya color para este valor se usa el Negativo
						result.append(([(ini[0],posicion),(dato[0],posicion)],tipo,grosor,colorNegativo))
				# Obtengo el color a usar
				color=find_color(rancol,dato[1])
				# Recuerdo el dato anterior
				ini=dato
			# Elementos que se hayan quedado descolgados
			if (color!=None):
				result.append(([(ini[0],posicion),(dato[0],posicion)],tipo,grosor,color))
			# }}}2
		# Devuelvo la lista {{{2
		return result
		# }}}2
	# }}}1
	# Cambia los None por NA {{{1
	## \internal
	## Intercambia los None por NA
	## \param self -
	## \param valor Valor a procesar
	## \return Valor identico pero si antes tenia None, ahora tiene un NA
	def None2NA(self,valor):
		if (valor==None):
			return self.NA()
		else:
			return valor
	# }}}1

# ### EXCEPCIONES ### #################################
# EXCEPTION CLASSES {{{1
# Basic exceptions
# Except (General Exception) {{{2
class Except(Exception):
	def __init__(self,string):
		self.string=string
	def __str__(self):
		return self.string
#}}}2
# IOError (Input/Output error) {{{2
class IOError(Exception):
	def __init__(self,string):
		self.string=string
	def __str__(self):
		return self.string
# }}}2
# Exceptions in calculadora_senales
# SyntaxError (Error in syntax of a command) {{{2
class SyntaxError(Exception):
	def __init__(self,string):
		self.string=string
	def __str__(self):
		return self.string
# }}}2
# }}}1
