#!/usr/bin/env python
#-*- coding: iso8859-15 -*-
##########################################################
#                     FILE_SFTP v1.0                     #
##########################################################
# Autor: Juan Miguel Taboada Godoy                       #
# Fecha: Szczecin, 05 de Septiembre de 2006              #
# Descripcin: Adquisitor de ficheros de servidor SFTP   #
# Versin: 2006090500                                    #
#                                                        #
# Codigo fuente bajo licencia GNU/GPL                    #
# Centrologic (Computational Logistic Center)            #
# http://www.centrologic.com - info@centrologic.com      #
##########################################################

# Libreras que voy a usar {{{1
import time
import datetime
import os
import shutil
import socket
import paramiko
from lib.DATE import tempo_limit
from lib.HERENCIA import BASE,DEFAULTCONFIG
# }}}1

# ### CONFIG ### ######################################
## Acceso a Telemando mediante SFTP: configuracion por defecto
class CONFIG(DEFAULTCONFIG):
	# Constructor {{{1
	def __init__(self):
		# Cargo las acciones del padre
		DEFAULTCONFIG.__init__(self,"FILE_SFTP")
	# }}}1
	# Config Telemando {{{1
	# download_descarga {{{2
	## Actualizar cada n-segundos
	## \param self -
	## \param arg Segundos entre acutalizaciones
	def download_tempo(self,arg=None):
		if (arg!=None):
			self._download_tempo=arg
		return self._download_tempo;
	# }}}2
	# server {{{2
	## Servidor SFTP
	## \param self -
	## \param arg Direccion IP o nombre del servidor SFTP
	def server(self,arg=None):
		if (arg!=None):
			self._server=arg
		return self._server;
	# }}}2
	# port {{{2
	## Puerto del servidor SFTP
	## \param self -
	## \param arg Puerto
	def port(self,arg=None):
		if (arg!=None):
			self._port=arg
		return self._port;
	# }}}2
	# user {{{2
	## Usuario de acceso
	## \param self -
	## \param arg Usuario de acceso al SFTP
	def user(self,arg=None):
		if (arg!=None):
			self._user=arg
		return self._user;
	# }}}2
	# passwd {{{2
	## Clave
	## \param self -
	## \param arg Clave del usuario para acceso por SFTP
	def passwd(self,arg=None):
		if (arg!=None):
			self._passwd=arg
		return self._passwd;
	# }}}2
	# recv {{{2
	## Directorio del servidor en el que estan los registros
	## \param self -
	## \param arg Directorio del servidor
	def recv(self,arg=None):
		if (arg!=None):
			self._recv=arg
		return self._recv
	# }}}2
	# save {{{2
	## Directorio del servidor en el que guardar una copia de los registros
	## \param self -
	## \param arg Directorio del servidor
	def save(self,arg=None):
		if (arg!=None):
			self._save=arg
		return self._save
	# }}}2
	# dirtemp {{{2
	## Directorio donde almacenar temporalmente los registros descargados
	## \param self -
	## \param arg Directorio local
	def dirtemp(self,arg=None):
		if (arg!=None):
			self._dirtemp=arg
		return self._dirtemp
	# }}}2
	# Guarda la llave RSA de SSH {{{2
	## Acceso a la llave RSA de SSH
	## \param self -
	## \param arg Nombre del fichero con la llave RSA de SSH
	def sshkey(self,arg=None):
		if (arg!=None):
			self._sshkey=arg
		return self._sshkey
	# }}}2
	# memoria {{{2
	## Fichero donde se almacena la memoria
	## \param self -
	## \param arg Nombre del fichero
	def memoria(self,arg=None):
		if (arg!=None):
			self._memoria=arg
		return self._memoria
	# }}}2
	# }}}1
	# Checker {{{1
	## Comprueba que el conjunto minimo de datos requeridos por la clase han sido declarados por el usuario: \n
	## - download_tempo()
	## - debug()
	## - recv()
	## - save()
	## - dirtemp()
	## - sshkey()
	## - memoria()
	## \param self -
	def check(self):
		self._check("download_tempo")
		self._check("debug")
		self._check("recv")
		self._check("save")
		self._check("dirtemp")
		self._check("sshkey")
		self._check("memoria")
		self.check_default()
	# }}}1

# ### TELEMANDO ### ###################################
## Acceso a Telemando mediante SFTP: Telemando
class TELEMANDO(BASE):
	# Constructor {{{1
	## Constructor
	## \param self -
	## \param id Identificador del Telemando (ALFANUMERICO)
	def __init__(self,id):
		# Cargo las acciones del padre
		BASE.__init__(self,id,"FILE_SFTP")
		# Inicializo el estado interno del Telemando
		self.__incongruencias=[]
	# }}}1

	# Guarda la config {{{1
	## Guarda la configuracion del Telemando
	## \param self -
	## \param config Configuracin por defecto obtenida de la clase CONFIG
	## \param server Direccion IP o nombre del servidor FTP
	## \param fichero Nombre del fichero a descargar
	## \param user Usuario del servidor FTP
	## \param password Clave del usuario FTP
	## \param port Puerto del servidor FTP
	def config(self,config,server,fichero,user=None,password=None,port=None):
		# Configuracin por defecto
		self.default_config(config)
		# Cargo los datos en la config
		self.server(server)
		self.fichero(fichero)
		if (user!=None):
			self.user(user)
		if (password!=None):
			self.passwd(password)
		if (port!=None):
			self.port(port)
		# Chequeo la config
		self.check()
	# }}}1
	# Guarda el server {{{1
	## Servidor SFTP
	## \param self -
	## \param arg Direccion IP o nombre del servidor SFTP
	def server(self,arg=None):
		if (arg!=None):
			self._server=arg
		return self._server
	# }}}1
	# Guarda el fichero {{{1
	## Fichero a descargar del servidor SFTP
	## \param self -
	## \param arg Nombre del fichero
	def fichero(self,arg=None):
		if (arg!=None):
			self._fichero=arg
		return self._fichero
	# }}}1
	# Guarda el usuario {{{1
	## Usuario de acceso
	## \param self -
	## \param arg Usuario de acceso al SFTP
	def user(self,arg=None):
		if (arg!=None):
			self._user=arg
		return self._user
	# }}}1
	# Guarda la clave {{{1
	## Clave
	## \param self -
	## \param arg Clave del usuario para acceso por SFTP
	def passwd(self,arg=None):
		if (arg!=None):
			self._passwd=arg
		return self._passwd
	# }}}1
	# Guarda el puerto {{{1
	## Puerto del servidor FTP
	## \param self -
	## \param arg Puerto
	def port(self,arg=None):
		if (arg!=None):
			self._port=arg
		return self._port
	# }}}1

	# Checker {{{1
	## Comprueba que el conjunto minimo de datos requeridos por la clase han sido declarados por el usuario: \n
	## - server()
	## - fichero()
	## \param self -
	def check(self):
		self._check("server")
		self._check("fichero")
		self.check_default()
	# }}}1

	# Descarga los datos del Telemando {{{1
	## \internal
	## Descarga los datos del Telemando
	## \param self -
	def download_extendido(self):
		# Creo el nombre del fichero origen {{{2
		origen=self.valor("fichero")
		# }}}2
		# Creo el nombre del fichero destino {{{2
		date=datetime.datetime.fromtimestamp(time.time())
		fecha=date.timetuple()
		nombre_fichero=self.valor("fichero")
		extension=nombre_fichero.split(".")
		destino="%s_%s.%s" % (self.internal_id(),tempo_limit(fecha,self.valor("download_tempo")),extension[-2])
		# }}}2
		self.debug("Inicia la descarga de datos [%s,%s]" % (origen,destino))
		# Cambio al directorio destino {{{2
		self.debug("Cambia al directorio %s" % (self.valor("dirtemp")))
		pwd=os.getcwd()
		try:
			os.chdir(self.valor("dirtemp"))
		except Exception:
			# Genero un error porque no puedo cambiarme al directorio destino
			raise IOError,"Error al cambiar al directorio \"%s\"" % (self.valor("dirtemp"))
		# Habilito una incongruencia
		self.habilitar(("sincronizar1",pwd))
		# }}}2
		# Descarga de los registros del Telemando fsico {{{2
		# Revisa si el registro existe {{{3
		self.debug("Comprobando existencia previa del registro")
		try:
			file_exists=open("bak/%s.dat" % (destino))
			# Cambio al directorio origen
			try:
				os.chdir(pwd)
			except Exception,e:
				# Genero un error porque no puedo cambiarme al directorio destino
				raise TerminalError,"No se ha podido volver al directorio origen: %s" % (e)
			# Desabilito la incongruencia
			self.deshabilitar(("sincronizar1",pwd,self.valor("dirtemp")))
		except:
			file_exists=0
		# }}}3
		# Test de existencia: Si no dispone del registo se lo baja
		if (not (file_exists)):
			# Conecto al SFTP {{{3
			self.debug("Conectando al SSH... ssh://%s:%s" % (self.valor("server"),self.valor("port")))
			try:
				ssh=paramiko.Transport("%s:%s" % (self.valor("server"),self.valor("port")))
			except socket.error,e:
				# Genero un error
				raise ConnectionError,"No se pudo conectar al servidor SFTP: %s" % e
			# }}}3
			# Abro la llave RSA {{{3
			self.debug("Abriendo la llave RSA '%s'" % (self.valor("sshkey")))
			try:
				key = paramiko.RSAKey.from_private_key_file("../../../%s" % (self.valor("sshkey")),self.valor("passwd"))
			except paramiko.PasswordRequiredException:
				raise IOError,"No es posible abrir la llave RSA (%s) con la clave configurada. Para crear una nueva llave ejecute el comando 'ssh-keygen -t rsa'" % (self.valor("sshkey"))
			except paramiko.SSHException:
				raise IOError,"No es posible abrir la llave RSA (%s) con la clave configurada. Para crear una nueva llave ejecute el comando 'ssh-keygen -t rsa'" % (self.valor("sshkey"))
			except Exception,e:
				raise IOError,"No es posible abrir la llave RSA (%s). Para crear una nueva llave ejecute el comando 'ssh-keygen -t rsa' (ERROR: %s)" % (self.valor("sshkey"),e)
			# }}}3
			# Autentificando {{{3
			self.debug("Autentificando con el usario '%s'" % (self.valor("user")))
			try:
				ssh.connect(username=self.valor("user"), pkey=key)
			except paramiko.SSHException:
				ssh.close()
				raise ConnectionError,"Error en la autentificacion: revise el server/usuario, o bien acepte la llave RSA en el servidor. Para aceptar su llave RSA, inserte el contenido la llave publica al authorized_keys del servidor"
			# }}}3
			# Obtengo acceso al protocolo SFTP {{{3
			self.debug("Inicio la capa de transporte SFTP")
			sftp=paramiko.SFTP.from_transport(ssh)
			# }}}3
			# Obtengo la hora actual {{{3
			self.debug("Obteniendo la hora")
			hora=time.time()
			# }}}3
			# Recolecto el registro del servidor {{{3
			self.debug("Recolectando el registro del servidor")
			try:
				self._recolectar_registro(sftp,self.valor("recv"),self.valor("save"),self.valor("fichero"),hora)
			except Exception,e:
				# Desbloqueo el servidor {{{4
				m="Registro no recolectado por error en el proceso: %s" % e
				try:
					self.debug("Cerrando la capa de transporte SFTP...")
					sftp.close()
				except Exception,e:
					m="%s\n%s" % (m,"No he podido cerrar la capa de transporte SFTP: %s" % (e))
				try:
					self.debug("Desconectando del servidor SSH...")
					ssh.close()
				except Exception,e:
					m="%s\n%s" % (m,"No he podido desconectar del servidor SSH: %s" % (e))
				# }}}4
				# Devuelvo un error de comunicaciones
				raise ConnectionError,m
			# }}}3
			# Desconecto del servidor SFTP sin importar lo que pase {{{3
			try:
				self.debug("Cerrando la capa de transporte SFTP...")
				sftp.close()
				self.debug("Desconectando del servidor SSH...")
				ssh.close()
			except Exception,e:
				self.debug("Ocultado error en Telemando %s al desconectar del servidor de FTP: %s" % (self.internal_id(),e))
			# }}}3
			# Procesa los datos descargados {{{3
			self.debug("Procesando los datos descargados")
			# Copio el fichero de registros al directorio de salida {{{4
			self.debug("Copiando el fichero de registros al directorio de salida")
			try:
				os.rename(origen,"%s.dat" % (destino))
				shutil.copyfile("%s.dat" % (destino),"dat/%s.dat" % (destino))
				shutil.copyfile("%s.dat" % (destino),"bak/%s.dat" % (destino))
				os.unlink("%s.dat" % (destino))
			except Exception,e:
				self.debug("Error en Telemando %s al manejar el fichero de registros %s. Error: %s" % (self.valor("id"),origen,e))
			# }}}4
			# }}}3
		else:
			self.debug("Cancelada descarga, no es necesario sincronizar")
			# Cierro el fichero abierto por el test de existencia {{{3
			try:
				file_exists.close()
			except Exception,e:
				self.debug("Ocultado error en Telemando %s al cerrar el fichero de registros tras la prueba de existencia de fichero. Error: %s" % (self.valor("id"),e))
			# }}}3
		# }}}2
		# Vuelvo al directorio origen {{{2
		self.debug("Volviendo al directorio %s" % (pwd))
		try:
			os.chdir(pwd)
		except Exception,e:
			# Genero un error porque no puedo cambiarme al directorio destino
			raise TerminalError,"No se ha podido volver al directorio origen: %s" % (e)
		# Desabilito la incongruencia
		self.deshabilitar(("sincronizar1",pwd,self.valor("dirtemp")))
		# }}}2
	# }}}1
	# Recolector del registro {{{1
	## \internal
	## Recolector del registro
	## \param self -
	## \param ftp Conector al servidor FTP
	## \param recv Directorio remoto en el que esta el registro a recolectar
	## \param save Directorio remoto en el que guardar una copia de seguridad
	## \param fichero Nombre del fichero a recolectar
	## \param hora Hora que poner al fichero local una vez recibido
	def _recolectar_registro(self,sftp,recv,save,fichero,hora):
		# Nombre de los ficheros
		local=fichero
		remoto="%s%s" % (recv,fichero)
		backup="%s%s_%s" % (save,int(hora),fichero)
		self.debug("   Usando como fichero local '%s'" % (local))
		self.debug("   Usando como fichero remoto '%s'" % (remoto))
		self.debug("   Usando como fichero de backup '%s'" % (backup))
		
		# Comprueba que el fichero remoto existe
		try:
			exists=sftp.open(remoto,"rb")
		except Exception:
			raise ConnectionError,"No existe el fichero '%s' en el servidor SSH" % (remoto)
		exists.close()
		
		# Abro el fichero local
		self.debug("Abriendo el fichero local '%s'" % (local))
		LOCAL=open(local,'w')
		# Copio los datos
		self.debug("Copiando datos del remoto '%s' al local" % (remoto))
		LOCAL.write(sftp.open(remoto,"rb").read())
		# Cierro los ficheros
		self.debug("Cerrando el fichero local")
		LOCAL.close()
		
		# Abro el fichro de backups
		self.debug("Abriendo el fichero de backups '%s'" % (backup))
		BACKUP=sftp.open(backup,'w')
		# Copio los datos al backup
		self.debug("Copiando datos del remoto '%s' al backup" % (remoto))
		BACKUP.write(sftp.open(remoto,"rb").read())
		# Cierro los ficheos
		self.debug("Cerrando el fichero de backups")
		BACKUP.close()
		
		# Borro el fichero remoto
		sftp.remove(remoto)
		
		# Cambio la fecha de acceso y de modificacin
		#sftp.utime("%s%s" % (directorio,nuevo_nombre),(atime,mtime))
	# }}}1
	# Control de descarga de los Telemandos {{{1
	## Control de descarga de los Telemandos (esta clase simplifica el proceso de recolectar Excepciones)
	## \param self -
	def download(self):
		self.debug("Descargando datos para el Telemando '%s'" % (self.internal_id()))
		try:
			self.download_extendido()
		except IOError,e:							# Error definitivo de entrada/salida
			# Si hay error al sincronizar, no se valida el TELEMANDO (podra disponer del resultado en la BD)
			# ANULADO: raise InvalidarTelemando, "%s" % (e)
			self.debug("El Telemando %s no ha sincronizado debido a un error de Entrada/Salida (%s)" % (self.internal_id(),e))
			# Necesito estabilizar el Telemando para seguir usndolo
			self.estabilizar()
			raise
		self.debug("")
	# }}}1

	# Mtodo para resolver incongruencias {{{1
	## \internal
	## Metodo para resolver incongruencias
	## \param self -
	def estabilizar(self):
		# Si hay incongruencias que resolver, las resuelvo
		for elemento in self.__incongruencias:
			# Sincronizar 1
			if (elemento[0]=="sincronizar1"):
				self.debug("Resolviendo incongruencia 'sincronizar1'")
				# Obtengo el directorio al que debo cambiar
				pwd=elemento[1]
				try:
					# Cambio al directorio
					os.chdir(pwd)
				except Exception,e:
					raise TerminalError,"ESTABILIZADOR: No se ha podido volver al directorio origen: %s" % (e)
			# Si es una incongruencia desconocida
			else:
				# La situacin es incongruente y no puede ser resuelta (lanzo una excepcin Terminal)
				raise TerminalError,"ESTABILIZADOR: Se han detectado incongruencias desconocidas"
		# Todas las incongruencias han sido resueltas (la lista queda vaca)
		self.__incongruencias=[]
	# }}}1
	# Mtodo para habilitar incongruencias {{{1
	## \internal
	## Metodo para habilitar incongruencias
	## \param self -
	## \param incongruencia ID Alfanumerico de la incongruencia
	def habilitar(self,incongruencia):
		# Las incongruencias las inserto al principio (mantengo lista invertida)
		self.debug("Activando control de incongruencia '%s'" % (incongruencia[0]))
		self.__incongruencias.insert(0,incongruencia)
	# }}}1
	# Mtodo para deshabilitar incongruencias {{{1
	## \internal
	## Metodo para deshabilitar incongruencias
	## \param self -
	## \param incongruencia ID Alfanumerico de la incongruencia
	def deshabilitar(self,incongruencia):
		# Inicializo una lista vaca
		temp=[]
		# Para toda la lista (manteniendo el orden establecido)
		for elemento in self.__incongruencias:
			# Me quedo con las que no coincidan
			if (elemento!=incongruencia):
				self.debug("Desactivando control de incongruencia '%s'" % (incongruencia[0]))
				# Cargo el elemento a la lista temporal
				temp.append(elemento)
		# Rehago las lista de incongruencias con la temporal
		self.__incongruencias=temp
	# }}}1

# EXCEPTION CLASSES {{{1
# Excepciones bsicas
# Except (General Exception) {{{2
class Except(Exception):
	def __init__(self,string):
		self.string=string
	def __str__(self):
		return self.string
#}}}2
# LockError (Server locked error) {{{2
class LockError(Exception):
	def __init__(self,string):
		self.string=string
	def __str__(self):
		return self.string
# }}}2
# ConnectionError (Conexin errnea al servidor) {{{2
class ConnectionError(Exception):
	def __init__(self,string):
		self.string=string
	def __str__(self):
		return self.string
# }}}2
# IOError (Error de entrada/salida) {{{2
class IOError(Exception):
	def __init__(self,string):
		self.string=string
	def __str__(self):
		return self.string
# }}}2
# ExecutionError (Error de ejecuccin) {{{2
class ExecutionError(Exception):
	def __init__(self,string):
		self.string=string
	def __str__(self):
		return self.string
# }}}2
# Excepciones graves
# TerminalError (Error irrecuperable del programa) {{{2
class TerminalError(Exception):
	def __init__(self,string):
		self.string=string
	def __str__(self):
		return self.string
# }}}2
# }}}1
